/* Automatically generated from impl/SVGPathSegLinetoHorizontalImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathSegLinetoHorizontalAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
};


static const struct HashEntry SVGPathSegLinetoHorizontalAbsImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegLinetoHorizontalAbsImpl::X, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegLinetoHorizontalAbsImpl::s_hashTable = { 2, 2, SVGPathSegLinetoHorizontalAbsImpl__s_hashTableEntries, 2, SVGPathSegLinetoHorizontalAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathSegLinetoHorizontalRelImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
};


static const struct HashEntry SVGPathSegLinetoHorizontalRelImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegLinetoHorizontalRelImpl::X, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegLinetoHorizontalRelImpl::s_hashTable = { 2, 2, SVGPathSegLinetoHorizontalRelImpl__s_hashTableEntries, 2, SVGPathSegLinetoHorizontalRelImpl__s_hashTableStrings};

} // namespace
