<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="djvumultipage" version="3">
  <MenuBar>

    <Menu name="edit"><text>&amp;Edit</text>
     <Separator/>
     <Action name="delete_pages"/>
    </Menu>

    <Menu name="view"><text>&amp;View</text>
      <Action name="render_mode"/>
    </Menu>

  </MenuBar>
</kpartgui>
