<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KView" version="12">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<DefineGroup name="open_merge_group" append="open_merge"/>
			<DefineGroup name="save_merge_group" append="save_merge"/>
			<Action name="print" append="print_merge"/>
			<!--<Merge /> This creates an extra separator in the file menu-->
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="crop" />
			<Action name="reload" />
			<Separator />
			<Action name="del" />
			<Merge />
		</Menu>
		<Merge />
		<Menu name="settings"><text>&amp;Settings</text>
			<DefineGroup name="show_merge_group" append="show_merge" />
			<Separator append="show_merge"/>
		</Menu>
		<Menu name="help"><text>&amp;Help</text>
			<DefineGroup name="about_merge_group" append="about_merge"/>
		</Menu>
	</MenuBar>
	<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
		<Action name="file_open"/>
		<Action name="file_save"/>
		<Action name="print"/>
		<Action name="file_print_preview"/>
		<Separator/>
		<Action name="edit_undo"/>
		<Action name="edit_redo"/>
		<Action name="edit_cut"/>
		<Action name="edit_copy"/>
		<Action name="edit_paste"/>
		<Separator/>
		<Action name="reload"/>
	</ToolBar>
	<Menu name="popupmenu">
		<Action name="file_open"/>
		<DefineGroup name="file_group"/>
		<Separator/>
		<Action name="options_show_menubar"/>
		<Action name="fullscreen"/>
		<Separator/>
		<Action name="del"/>
		<Merge/>
	</Menu>
</kpartgui>
