/* Automatically generated from impl/SVGImageElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGImageElementImpl__s_hashTableStrings[] = {
    "\0"
    "preserveAspectRatio\0"
    "height\0"
    "width\0"
    "href\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGImageElementImpl__s_hashTableEntries[] = {
   { 1, SVGImageElementImpl::PreserveAspectRatio, DontDelete|ReadOnly, 0, -1 },
   { 39, SVGImageElementImpl::X, DontDelete|ReadOnly, 0, 7 },
   { 41, SVGImageElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 21, SVGImageElementImpl::Height, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 28, SVGImageElementImpl::Width, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 34, SVGImageElementImpl::Href, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGImageElementImpl::s_hashTable = { 2, 8, SVGImageElementImpl__s_hashTableEntries, 7, SVGImageElementImpl__s_hashTableStrings};

} // namespace
