/* Automatically generated from impl/SVGTestsImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGTestsImpl__s_hashTableStrings[] = {
    "\0"
    "requiredExtensions\0"
    "requiredFeatures\0"
    "systemLanguage\0"
};


static const struct HashEntry SVGTestsImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 20, SVGTestsImpl::RequiredFeatures, DontDelete|ReadOnly, 0, 5 },
   { 1, SVGTestsImpl::RequiredExtensions, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 37, SVGTestsImpl::SystemLanguage, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGTestsImpl::s_hashTable = { 2, 6, SVGTestsImpl__s_hashTableEntries, 5, SVGTestsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGTestsImplProto__s_hashTableStrings[] = {
    "\0"
    "hasExtension\0"
};


static const struct HashEntry SVGTestsImplProto__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGTestsImpl::HasExtension, DontDelete|Function, 1, -1 }
};

const struct HashTable SVGTestsImplProto::s_hashTable = { 2, 2, SVGTestsImplProto__s_hashTableEntries, 2, SVGTestsImplProto__s_hashTableStrings};

} // namespace
