/* Automatically generated from impl/SVGStopElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGStopElementImpl__s_hashTableStrings[] = {
    "\0"
    "stop-opacity\0"
    "offset\0"
};


static const struct HashEntry SVGStopElementImpl__s_hashTableEntries[] = {
   { 1, SVGStopElementImpl::StopOpacity, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 14, SVGStopElementImpl::Offset, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGStopElementImpl::s_hashTable = { 2, 3, SVGStopElementImpl__s_hashTableEntries, 3, SVGStopElementImpl__s_hashTableStrings};

} // namespace
