/* Automatically generated from impl/SVGRectImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGRectImpl__s_hashTableStrings[] = {
    "\0"
    "height\0"
    "width\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGRectImpl__s_hashTableEntries[] = {
   { 14, SVGRectImpl::X, DontDelete, 0, -1 },
   { 16, SVGRectImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGRectImpl::Height, DontDelete, 0, -1 },
   { 8, SVGRectImpl::Width, DontDelete, 0, -1 }
};

const struct HashTable SVGRectImpl::s_hashTable = { 2, 5, SVGRectImpl__s_hashTableEntries, 5, SVGRectImpl__s_hashTableStrings};

} // namespace
