<!DOCTYPE kpartgui>
<kpartgui name="KGhostViewShell" version="11">
<State name="initState">
    <Disable>
       <Action name="file_print" />
       <Action name="reload" />
       <Action name="maximize"/>
       <Action name="fullscreen"/>
       <Action name="showmenubar"/>
    </Disable>
</State>
<State name="documentState">
    <Enable>
       <Action name="file_print" />
       <Action name="reload" />
       <Action name="fullscreen"/>
       <Action name="showmenubar"/>
       <Action name="maximize"/>
    </Enable>
</State>
<MenuBar>
  <Menu name="file">
    <DefineGroup name="file_save" append="save_merge" />
    <DefineGroup name="file_print" append="print_merge" />
  </Menu>
  <Menu name="edit">
    <MergeLocal/>
  </Menu>
  <Menu name="view">
    <Action name="reload" />
    <Action name="maximize"/>
    <Action name="fullscreen"/>
  </Menu>
  <Menu name="settings">
    <Action name="option_show_toolbar" />
  </Menu>
  <Merge/>
</MenuBar>
<ToolBar name="mainToolBar">
  <text>&amp;Main Toolbar</text>
</ToolBar>
</kpartgui>

