<!DOCTYPE kpartgui>
<kpartplugin name="kview_scale" library="kview_scale">
	<MenuBar>
		<Menu name="image"><Text>&amp;Image</Text>
			<Action name="plugin_scale"/>
		</Menu>
	</MenuBar>
	<!--
	<ToolBar name="extraToolBar">
        <text>&amp;Extra Toolbar</text>
		<Action name="plugin_scale"/>
	</ToolBar>
	-->
</kpartplugin>
