/* Automatically generated from impl/SVGPointImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPointImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPointImpl__s_hashTableEntries[] = {
   { 1, SVGPointImpl::X, DontDelete, 0, -1 },
   { 3, SVGPointImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPointImpl::s_hashTable = { 2, 3, SVGPointImpl__s_hashTableEntries, 3, SVGPointImpl__s_hashTableStrings};

} // namespace
