/* Automatically generated from impl/SVGGlyphElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGGlyphElementImpl__s_hashTableStrings[] = {
    "\0"
    "vert-origin-x\0"
    "vert-origin-y\0"
    "arabic-form\0"
    "horiz-adv-x\0"
    "orientation\0"
    "glyph-name\0"
    "vert-adv-y\0"
    "unicode\0"
    "lang\0"
    "d\0"
};


static const struct HashEntry SVGGlyphElementImpl__s_hashTableEntries[] = {
   { 95, SVGGlyphElementImpl::Lang, DontDelete|ReadOnly, 0, -1 },
   { 100, SVGGlyphElementImpl::D, DontDelete|ReadOnly, 0, -1 },
   { 29, SVGGlyphElementImpl::ArabicForm, DontDelete|ReadOnly, 0, -1 },
   { 41, SVGGlyphElementImpl::HorizAdvX, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 87, SVGGlyphElementImpl::Unicode, DontDelete|ReadOnly, 0, -1 },
   { 76, SVGGlyphElementImpl::VertAdvY, DontDelete|ReadOnly, 0, -1 },
   { 53, SVGGlyphElementImpl::Orientation, DontDelete|ReadOnly, 0, -1 },
   { 65, SVGGlyphElementImpl::GlyphName, DontDelete|ReadOnly, 0, 11 },
   { 15, SVGGlyphElementImpl::VertOriginY, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGGlyphElementImpl::VertOriginX, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGGlyphElementImpl::s_hashTable = { 2, 12, SVGGlyphElementImpl__s_hashTableEntries, 11, SVGGlyphElementImpl__s_hashTableStrings};

} // namespace
