/* Automatically generated from impl/SVGAnimatedAngleImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGAnimatedAngleImpl__s_hashTableStrings[] = {
    "\0"
    "animVal\0"
    "baseVal\0"
};


static const struct HashEntry SVGAnimatedAngleImpl__s_hashTableEntries[] = {
   { 9, SVGAnimatedAngleImpl::BaseVal, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGAnimatedAngleImpl::AnimVal, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGAnimatedAngleImpl::s_hashTable = { 2, 3, SVGAnimatedAngleImpl__s_hashTableEntries, 3, SVGAnimatedAngleImpl__s_hashTableStrings};

} // namespace
