<!DOCTYPE kpartgui>
<kpartgui name="kdvi_part" version="4">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
  <Action name="info_dvi" group="info_merge"/>
  <Menu name="export" group="print_merge" ><text>Export As</text>
   <Action name="export_postscript" group="export_merge"/>
   <Action name="export_pdf" group="export_merge"/>
  </Menu>
 </Menu>

 <Menu name="edit"><text>&amp;Edit</text>
   <Separator/>
   <Action name="embed_postscript"/>
 </Menu>

 <Menu name="settings"><text>&amp;Settings</text>
  <Action name="enable_msgs" group="additional_settings_merge"/>
 </Menu>

 <Menu name="help"><text>&amp;Help</text>
  <Action name="help_tipofday" group="help_top"/>
 </Menu>
</MenuBar>

</kpartgui>
