/* Automatically generated from impl/SVGAnimationElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGAnimationElementImpl__s_hashTableStrings[] = {
    "\0"
    "attributeName\0"
    "attributeType\0"
    "targetElement\0"
    "repeatCount\0"
    "accumulate\0"
    "keySplines\0"
    "repeatDur\0"
    "additive\0"
    "calcMode\0"
    "keyTimes\0"
    "restart\0"
    "values\0"
    "begin\0"
    "fill\0"
    "from\0"
    "href\0"
    "dur\0"
    "end\0"
    "max\0"
    "min\0"
    "by\0"
    "to\0"
};


static const struct HashEntry SVGAnimationElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 55, SVGAnimationElementImpl::Accumulate, DontDelete|ReadOnly, 0, 25 },
   { 0, 0, 0, 0, -1 },
   { 29, SVGAnimationElementImpl::TargetElement, DontDelete|ReadOnly, 0, 26 },
   { 66, SVGAnimationElementImpl::KeySplines, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 145, SVGAnimationElementImpl::Href, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGAnimationElementImpl::AttributeName, DontDelete|ReadOnly, 0, -1 },
   { 150, SVGAnimationElementImpl::Dur, DontDelete|ReadOnly, 0, 30 },
   { 96, SVGAnimationElementImpl::CalcMode, DontDelete|ReadOnly, 0, -1 },
   { 129, SVGAnimationElementImpl::Begin, DontDelete|ReadOnly, 0, -1 },
   { 122, SVGAnimationElementImpl::Values, DontDelete|ReadOnly, 0, 23 },
   { 0, 0, 0, 0, -1 },
   { 87, SVGAnimationElementImpl::Additive, DontDelete|ReadOnly, 0, 27 },
   { 105, SVGAnimationElementImpl::KeyTimes, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 15, SVGAnimationElementImpl::AttributeType, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 169, SVGAnimationElementImpl::To, DontDelete|ReadOnly, 0, 29 },
   { 0, 0, 0, 0, -1 },
   { 140, SVGAnimationElementImpl::From, DontDelete|ReadOnly, 0, -1 },
   { 166, SVGAnimationElementImpl::By, DontDelete|ReadOnly, 0, 24 },
   { 154, SVGAnimationElementImpl::End, DontDelete|ReadOnly, 0, 28 },
   { 162, SVGAnimationElementImpl::Min, DontDelete|ReadOnly, 0, -1 },
   { 158, SVGAnimationElementImpl::Max, DontDelete|ReadOnly, 0, -1 },
   { 114, SVGAnimationElementImpl::Restart, DontDelete|ReadOnly, 0, -1 },
   { 43, SVGAnimationElementImpl::RepeatCount, DontDelete|ReadOnly, 0, -1 },
   { 77, SVGAnimationElementImpl::RepeatDur, DontDelete|ReadOnly, 0, -1 },
   { 135, SVGAnimationElementImpl::Fill, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGAnimationElementImpl::s_hashTable = { 2, 31, SVGAnimationElementImpl__s_hashTableEntries, 23, SVGAnimationElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGAnimationElementImplProto__s_hashTableStrings[] = {
    "\0"
    "getSimpleDuration\0"
    "getCurrentTime\0"
    "getStartTime\0"
};


static const struct HashEntry SVGAnimationElementImplProto__s_hashTableEntries[] = {
   { 34, SVGAnimationElementImpl::GetStartTime, DontDelete|Function, 0, -1 },
   { 1, SVGAnimationElementImpl::GetSimpleDuration, DontDelete|Function, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 19, SVGAnimationElementImpl::GetCurrentTime, DontDelete|Function, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGAnimationElementImplProto::s_hashTable = { 2, 5, SVGAnimationElementImplProto__s_hashTableEntries, 5, SVGAnimationElementImplProto__s_hashTableStrings};

} // namespace
