<!DOCTYPE kpartgui>
<kpartgui version="2" name="kfax">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_add_fax" append="open_merge"/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="view_rotate"/>
    <Action name="view_mirror"/>
    <Action name="view_flip"/>
  </Menu>
</MenuBar>
 <ToolBar noMerge="1" name="mainToolBar" >
  <text>Main Toolbar</text>
  <Action name="file_open" />
  <Action name="file_save" />
  <Action name="file_print" />
  <Separator name="separator_2" />
  <Action name="go_previous" />
  <Action name="go_next" />
  <Separator name="separator_1" />
  <Action name="go_first" />
  <Action name="go_last" />
  <Separator name="separator_0" />
  <Action name="view_zoom_in" />
  <Action name="view_zoom_out" />
 </ToolBar>
</kpartgui>
