/* Automatically generated from impl/SVGGlyphRefElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGGlyphRefElementImpl__s_hashTableStrings[] = {
    "\0"
    "glyphRef\0"
    "format\0"
    "dx\0"
    "dy\0"
};


static const struct HashEntry SVGGlyphRefElementImpl__s_hashTableEntries[] = {
   { 1, SVGGlyphRefElementImpl::GlyphRef, DontDelete|ReadOnly, 0, -1 },
   { 18, SVGGlyphRefElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 21, SVGGlyphRefElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 17, SVGGlyphRefElementImpl::Dx, DontDelete|ReadOnly, 0, -1 },
   { 20, SVGGlyphRefElementImpl::Dy, DontDelete|ReadOnly, 0, -1 },
   { 10, SVGGlyphRefElementImpl::Format, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGGlyphRefElementImpl::s_hashTable = { 2, 7, SVGGlyphRefElementImpl__s_hashTableEntries, 7, SVGGlyphRefElementImpl__s_hashTableStrings};

} // namespace
