<!DOCTYPE kpartplugin>
<kpartplugin name="PhotoBook Part" library="libphotobook" version="2">
<MenuBar>
	<Menu name="view"><text>&amp;View</text>
		<Action name="previous"/>
		<Action name="next"/>
	</Menu>
</MenuBar>

<Toolbar name="go"><text>Go</text>
	<Action name="previous"/>
	<Action name="next"/>
</Toolbar>
</kpartplugin>
