/* Automatically generated from impl/SVGPathElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathElementImpl__s_hashTableStrings[] = {
    "\0"
    "pathLength\0"
    "d\0"
};


static const struct HashEntry SVGPathElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 12, SVGPathElementImpl::D, DontDelete|ReadOnly, 0, 3 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathElementImpl::PathLength, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPathElementImpl::s_hashTable = { 2, 4, SVGPathElementImpl__s_hashTableEntries, 3, SVGPathElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathElementImplProto__s_hashTableStrings[] = {
    "\0"
    "createSVGPathSegCurvetoQuadraticAbs\0"
    "createSVGPathSegCurvetoQuadraticRel\0"
    "createSVGPathSegLinetoHorizontalAbs\0"
    "createSVGPathSegLinetoHorizontalRel\0"
    "createSVGPathSegLinetoVerticalAbs\0"
    "createSVGPathSegLinetoVerticalRel\0"
    "createSVGPathSegCurvetoCubicAbs\0"
    "createSVGPathSegCurvetoCubicRel\0"
    "createSVGPathSegClosePath\0"
    "createSVGPathSegLinetoAbs\0"
    "createSVGPathSegLinetoRel\0"
    "createSVGPathSegMovetoAbs\0"
    "createSVGPathSegMovetoRel\0"
    "createSVGPathSegArcAbs\0"
    "createSVGPathSegArcRel\0"
    "getPathSegAtLength\0"
    "getPointAtLength\0"
    "getTotalLength\0"
};


static const struct HashEntry SVGPathElementImplProto__s_hashTableEntries[] = {
   { 472, SVGPathElementImpl::GetPointAtLength, DontDelete|Function, 1, -1 },
   { 453, SVGPathElementImpl::GetPathSegAtLength, DontDelete|Function, 1, 23 },
   { 0, 0, 0, 0, -1 },
   { 355, SVGPathElementImpl::CreateSVGPathSegMovetoAbs, DontDelete|Function, 2, -1 },
   { 1, SVGPathElementImpl::CreateSVGPathSegCurvetoQuadraticAbs, DontDelete|Function, 4, 28 },
   { 430, SVGPathElementImpl::CreateSVGPathSegArcRel, DontDelete|Function, 7, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 109, SVGPathElementImpl::CreateSVGPathSegLinetoHorizontalRel, DontDelete|Function, 1, 25 },
   { 0, 0, 0, 0, -1 },
   { 303, SVGPathElementImpl::CreateSVGPathSegLinetoAbs, DontDelete|Function, 2, -1 },
   { 0, 0, 0, 0, -1 },
   { 277, SVGPathElementImpl::CreateSVGPathSegClosePath, DontDelete|Function, 0, -1 },
   { 245, SVGPathElementImpl::CreateSVGPathSegCurvetoCubicRel, DontDelete|Function, 6, 27 },
   { 407, SVGPathElementImpl::CreateSVGPathSegArcAbs, DontDelete|Function, 7, -1 },
   { 381, SVGPathElementImpl::CreateSVGPathSegMovetoRel, DontDelete|Function, 2, -1 },
   { 37, SVGPathElementImpl::CreateSVGPathSegCurvetoQuadraticRel, DontDelete|Function, 4, 29 },
   { 0, 0, 0, 0, -1 },
   { 73, SVGPathElementImpl::CreateSVGPathSegLinetoHorizontalAbs, DontDelete|Function, 1, -1 },
   { 489, SVGPathElementImpl::GetTotalLength, DontDelete|Function, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 179, SVGPathElementImpl::CreateSVGPathSegLinetoVerticalRel, DontDelete|Function, 1, -1 },
   { 329, SVGPathElementImpl::CreateSVGPathSegLinetoRel, DontDelete|Function, 2, 24 },
   { 213, SVGPathElementImpl::CreateSVGPathSegCurvetoCubicAbs, DontDelete|Function, 6, 26 },
   { 145, SVGPathElementImpl::CreateSVGPathSegLinetoVerticalAbs, DontDelete|Function, 1, -1 },
   { 213, SVGPathElementImpl::CreateSVGPathSegCurvetoCubicAbs, DontDelete|Function, 4, -1 },
   { 245, SVGPathElementImpl::CreateSVGPathSegCurvetoCubicRel, DontDelete|Function, 4, -1 },
   { 1, SVGPathElementImpl::CreateSVGPathSegCurvetoQuadraticAbs, DontDelete|Function, 2, -1 },
   { 37, SVGPathElementImpl::CreateSVGPathSegCurvetoQuadraticRel, DontDelete|Function, 2, -1 }
};

const struct HashTable SVGPathElementImplProto::s_hashTable = { 2, 30, SVGPathElementImplProto__s_hashTableEntries, 23, SVGPathElementImplProto__s_hashTableStrings};

} // namespace
