<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="Kooka" version="3">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
   <Action name="foldernew"/>
   <Action name="saveImage"/>
   <Action name="importImage"/>
   <Action name="deleteImage"/>
   <Action name="unloadImage"/>
   <Action name="saveOCRResult"/>
 </Menu>
 <Menu name="imgCanvas"><text>&amp;Image</text>
   <Action name="openInGraphApp"/>
   <Separator/>
   <Action name="ocrImage"/>
   <Action name="ocrImageSelect"/>
   <Separator/>
   <Action name="scaleToWidth"/>
   <Action name="scaleToHeight"/>
   <Action name="scaleOriginal"/>
   <Separator/>
   <Action name="createFromSelection"/>
   <Separator/>
   <Action name="mirrorVertical"/>
   <Action name="mirrorHorizontal"/>
   <Action name="mirrorBoth"/>
   <Separator/>
   <Action name="rotateClockwise"/>
   <Action name="rotateCounterClockwise"/>
   <Action name="upsitedown"/>
 </Menu>
 <Menu name="settings"><text>&amp;Settings</text>
   <Action name="settings_show_docks" append="show_merge"/>
   <Action name="enable_msgs" append="save_merge"/>
   <Separator append="save_merge"/>
   <Action name="selectsource" append="save_merge"/>
   <Separator  append="save_merge"/>
   <Action name="loadscanparam" append="save_merge"/>
   <Action name="savescanparam" append="save_merge"/>
 </Menu>
</MenuBar>
 <ToolBar  name="mainToolBar" fullWidth="true">
   <text>Image Viewer Toolbar</text>
   <Action name="ocrImage"/>
   <Action name="ocrImageSelect"/>
   <Separator/>
   <Action name="scaleToWidth"/>
   <Action name="scaleToHeight"/>
   <Action name="scaleOriginal"/>
   <Action name="keepZoom"/>
   <Separator/>
   <Action name="createFromSelection"/>
   <Separator/>
   <Action name="mirrorVertical"/>
   <Action name="mirrorHorizontal"/>
   <Action name="mirrorBoth"/>
   <Separator/>
   <Action name="rotateClockwise"/>
   <Action name="rotateCounterClockwise"/>
   <Action name="upsitedown"/>
 </ToolBar>
</kpartgui>
