/* Automatically generated from impl/SVGPreserveAspectRatioImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPreserveAspectRatioImpl__s_hashTableStrings[] = {
    "\0"
    "meetOrSlice\0"
    "align\0"
};


static const struct HashEntry SVGPreserveAspectRatioImpl__s_hashTableEntries[] = {
   { 1, SVGPreserveAspectRatioImpl::MeetOrSlice, DontDelete, 0, -1 },
   { 13, SVGPreserveAspectRatioImpl::Align, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPreserveAspectRatioImpl::s_hashTable = { 2, 3, SVGPreserveAspectRatioImpl__s_hashTableEntries, 3, SVGPreserveAspectRatioImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPreserveAspectRatioImplConstructor__s_hashTableStrings[] = {
    "\0"
    "SVG_PRESERVEASPECTRATIO_XMAXYMAX\0"
    "SVG_PRESERVEASPECTRATIO_XMAXYMID\0"
    "SVG_PRESERVEASPECTRATIO_XMAXYMIN\0"
    "SVG_PRESERVEASPECTRATIO_XMIDYMAX\0"
    "SVG_PRESERVEASPECTRATIO_XMIDYMID\0"
    "SVG_PRESERVEASPECTRATIO_XMIDYMIN\0"
    "SVG_PRESERVEASPECTRATIO_XMINYMAX\0"
    "SVG_PRESERVEASPECTRATIO_XMINYMID\0"
    "SVG_PRESERVEASPECTRATIO_XMINYMIN\0"
    "SVG_PRESERVEASPECTRATIO_UNKNOWN\0"
    "SVG_PRESERVEASPECTRATIO_NONE\0"
    "SVG_MEETORSLICE_UNKNOWN\0"
    "SVG_MEETORSLICE_SLICE\0"
    "SVG_MEETORSLICE_MEET\0"
};


static const struct HashEntry SVGPreserveAspectRatioImplConstructor__s_hashTableEntries[] = {
   { 67, KSVG::SVG_PRESERVEASPECTRATIO_XMAXYMIN, DontDelete|ReadOnly, 0, 18 },
   { 0, 0, 0, 0, -1 },
   { 1, KSVG::SVG_PRESERVEASPECTRATIO_XMAXYMAX, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 166, KSVG::SVG_PRESERVEASPECTRATIO_XMIDYMIN, DontDelete|ReadOnly, 0, 17 },
   { 0, 0, 0, 0, -1 },
   { 34, KSVG::SVG_PRESERVEASPECTRATIO_XMAXYMID, DontDelete|ReadOnly, 0, 19 },
   { 0, 0, 0, 0, -1 },
   { 330, KSVG::SVG_PRESERVEASPECTRATIO_NONE, DontDelete|ReadOnly, 0, -1 },
   { 298, KSVG::SVG_PRESERVEASPECTRATIO_UNKNOWN, DontDelete|ReadOnly, 0, 20 },
   { 383, KSVG::SVG_MEETORSLICE_SLICE, DontDelete|ReadOnly, 0, -1 },
   { 133, KSVG::SVG_PRESERVEASPECTRATIO_XMIDYMID, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 265, KSVG::SVG_PRESERVEASPECTRATIO_XMINYMIN, DontDelete|ReadOnly, 0, -1 },
   { 359, KSVG::SVG_MEETORSLICE_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 232, KSVG::SVG_PRESERVEASPECTRATIO_XMINYMID, DontDelete|ReadOnly, 0, -1 },
   { 199, KSVG::SVG_PRESERVEASPECTRATIO_XMINYMAX, DontDelete|ReadOnly, 0, -1 },
   { 100, KSVG::SVG_PRESERVEASPECTRATIO_XMIDYMAX, DontDelete|ReadOnly, 0, -1 },
   { 405, KSVG::SVG_MEETORSLICE_MEET, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPreserveAspectRatioImplConstructor::s_hashTable = { 2, 21, SVGPreserveAspectRatioImplConstructor__s_hashTableEntries, 17, SVGPreserveAspectRatioImplConstructor__s_hashTableStrings};

} // namespace
