/* Automatically generated from impl/SVGAnimatedRectImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGAnimatedRectImpl__s_hashTableStrings[] = {
    "\0"
    "animVal\0"
    "baseVal\0"
};


static const struct HashEntry SVGAnimatedRectImpl__s_hashTableEntries[] = {
   { 9, SVGAnimatedRectImpl::BaseVal, DontDelete, 0, -1 },
   { 1, SVGAnimatedRectImpl::AnimVal, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGAnimatedRectImpl::s_hashTable = { 2, 3, SVGAnimatedRectImpl__s_hashTableEntries, 3, SVGAnimatedRectImpl__s_hashTableStrings};

} // namespace
