/* Automatically generated from impl/SVGCircleElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGCircleElementImpl__s_hashTableStrings[] = {
    "\0"
    "cx\0"
    "cy\0"
    "r\0"
};


static const struct HashEntry SVGCircleElementImpl__s_hashTableEntries[] = {
   { 4, SVGCircleElementImpl::Cy, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGCircleElementImpl::Cx, DontDelete|ReadOnly, 0, 5 },
   { 7, SVGCircleElementImpl::R, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGCircleElementImpl::s_hashTable = { 2, 6, SVGCircleElementImpl__s_hashTableEntries, 5, SVGCircleElementImpl__s_hashTableStrings};

} // namespace
