/* Automatically generated from impl/SVGAnimatedLengthListImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGAnimatedLengthListImpl__s_hashTableStrings[] = {
    "\0"
    "animVal\0"
    "baseVal\0"
};


static const struct HashEntry SVGAnimatedLengthListImpl__s_hashTableEntries[] = {
   { 9, SVGAnimatedLengthListImpl::BaseVal, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGAnimatedLengthListImpl::AnimVal, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGAnimatedLengthListImpl::s_hashTable = { 2, 3, SVGAnimatedLengthListImpl__s_hashTableEntries, 3, SVGAnimatedLengthListImpl__s_hashTableStrings};

} // namespace
