/* Automatically generated from impl/SVGPathSegCurvetoCubicImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathSegCurvetoCubicAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x1\0"
    "x2\0"
    "y1\0"
    "y2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoCubicAbsImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 13, SVGPathSegCurvetoCubicAbsImpl::X, DontDelete, 0, 7 },
   { 15, SVGPathSegCurvetoCubicAbsImpl::Y, DontDelete, 0, 8 },
   { 10, SVGPathSegCurvetoCubicAbsImpl::Y2, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoCubicAbsImpl::X1, DontDelete, 0, -1 },
   { 7, SVGPathSegCurvetoCubicAbsImpl::Y1, DontDelete, 0, 9 },
   { 4, SVGPathSegCurvetoCubicAbsImpl::X2, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoCubicAbsImpl::s_hashTable = { 2, 10, SVGPathSegCurvetoCubicAbsImpl__s_hashTableEntries, 7, SVGPathSegCurvetoCubicAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathSegCurvetoCubicRelImpl__s_hashTableStrings[] = {
    "\0"
    "x1\0"
    "x2\0"
    "y1\0"
    "y2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoCubicRelImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 13, SVGPathSegCurvetoCubicRelImpl::X, DontDelete, 0, 7 },
   { 15, SVGPathSegCurvetoCubicRelImpl::Y, DontDelete, 0, 8 },
   { 10, SVGPathSegCurvetoCubicRelImpl::Y2, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoCubicRelImpl::X1, DontDelete, 0, -1 },
   { 7, SVGPathSegCurvetoCubicRelImpl::Y1, DontDelete, 0, 9 },
   { 4, SVGPathSegCurvetoCubicRelImpl::X2, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoCubicRelImpl::s_hashTable = { 2, 10, SVGPathSegCurvetoCubicRelImpl__s_hashTableEntries, 7, SVGPathSegCurvetoCubicRelImpl__s_hashTableStrings};

} // namespace
