/* Automatically generated from impl/SVGPathSegLinetoImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathSegLinetoAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegLinetoAbsImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegLinetoAbsImpl::X, DontDelete, 0, -1 },
   { 3, SVGPathSegLinetoAbsImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegLinetoAbsImpl::s_hashTable = { 2, 3, SVGPathSegLinetoAbsImpl__s_hashTableEntries, 3, SVGPathSegLinetoAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathSegLinetoRelImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegLinetoRelImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegLinetoRelImpl::X, DontDelete, 0, -1 },
   { 3, SVGPathSegLinetoRelImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegLinetoRelImpl::s_hashTable = { 2, 3, SVGPathSegLinetoRelImpl__s_hashTableEntries, 3, SVGPathSegLinetoRelImpl__s_hashTableStrings};

} // namespace
