/* Automatically generated from impl/SVGForeignObjectElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGForeignObjectElementImpl__s_hashTableStrings[] = {
    "\0"
    "height\0"
    "width\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGForeignObjectElementImpl__s_hashTableEntries[] = {
   { 14, SVGForeignObjectElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 16, SVGForeignObjectElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGForeignObjectElementImpl::Height, DontDelete|ReadOnly, 0, -1 },
   { 8, SVGForeignObjectElementImpl::Width, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGForeignObjectElementImpl::s_hashTable = { 2, 5, SVGForeignObjectElementImpl__s_hashTableEntries, 5, SVGForeignObjectElementImpl__s_hashTableStrings};

} // namespace
