/* Automatically generated from impl/SVGScriptElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGScriptElementImpl__s_hashTableStrings[] = {
    "\0"
    "type\0"
};


static const struct HashEntry SVGScriptElementImpl__s_hashTableEntries[] = {
   { 1, SVGScriptElementImpl::Type, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGScriptElementImpl::s_hashTable = { 2, 2, SVGScriptElementImpl__s_hashTableEntries, 2, SVGScriptElementImpl__s_hashTableStrings};

} // namespace
