/* Automatically generated from impl/SVGZoomAndPanImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGZoomAndPanImpl__s_hashTableStrings[] = {
    "\0"
    "zoomAndPan\0"
};


static const struct HashEntry SVGZoomAndPanImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGZoomAndPanImpl::ZoomAndPan, DontDelete, 0, -1 }
};

const struct HashTable SVGZoomAndPanImpl::s_hashTable = { 2, 2, SVGZoomAndPanImpl__s_hashTableEntries, 2, SVGZoomAndPanImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGZoomAndPanImplConstructor__s_hashTableStrings[] = {
    "\0"
    "SVG_ZOOMANDPAN_DISABLE\0"
    "SVG_ZOOMANDPAN_MAGNIFY\0"
    "SVG_ZOOMANDPAN_UNKNOWN\0"
};


static const struct HashEntry SVGZoomAndPanImplConstructor__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 24, KSVG::SVG_ZOOMANDPAN_MAGNIFY, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 47, KSVG::SVG_ZOOMANDPAN_UNKNOWN, DontDelete|ReadOnly, 0, 5 },
   { 1, KSVG::SVG_ZOOMANDPAN_DISABLE, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGZoomAndPanImplConstructor::s_hashTable = { 2, 6, SVGZoomAndPanImplConstructor__s_hashTableEntries, 5, SVGZoomAndPanImplConstructor__s_hashTableStrings};

} // namespace
