/* Automatically generated from impl/SVGLineElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGLineElementImpl__s_hashTableStrings[] = {
    "\0"
    "x1\0"
    "x2\0"
    "y1\0"
    "y2\0"
};


static const struct HashEntry SVGLineElementImpl__s_hashTableEntries[] = {
   { 7, SVGLineElementImpl::Y1, DontDelete|ReadOnly, 0, 5 },
   { 10, SVGLineElementImpl::Y2, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGLineElementImpl::X1, DontDelete|ReadOnly, 0, -1 },
   { 4, SVGLineElementImpl::X2, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGLineElementImpl::s_hashTable = { 2, 6, SVGLineElementImpl__s_hashTableEntries, 5, SVGLineElementImpl__s_hashTableStrings};

} // namespace
