/* Automatically generated from impl/SVGPathSegCurvetoQuadraticImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathSegCurvetoQuadraticAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x1\0"
    "y1\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoQuadraticAbsImpl__s_hashTableEntries[] = {
   { 7, SVGPathSegCurvetoQuadraticAbsImpl::X, DontDelete, 0, 5 },
   { 9, SVGPathSegCurvetoQuadraticAbsImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoQuadraticAbsImpl::X1, DontDelete, 0, -1 },
   { 4, SVGPathSegCurvetoQuadraticAbsImpl::Y1, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoQuadraticAbsImpl::s_hashTable = { 2, 6, SVGPathSegCurvetoQuadraticAbsImpl__s_hashTableEntries, 5, SVGPathSegCurvetoQuadraticAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathSegCurvetoQuadraticRelImpl__s_hashTableStrings[] = {
    "\0"
    "x1\0"
    "y1\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoQuadraticRelImpl__s_hashTableEntries[] = {
   { 7, SVGPathSegCurvetoQuadraticRelImpl::X, DontDelete, 0, 5 },
   { 9, SVGPathSegCurvetoQuadraticRelImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoQuadraticRelImpl::X1, DontDelete, 0, -1 },
   { 4, SVGPathSegCurvetoQuadraticRelImpl::Y1, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoQuadraticRelImpl::s_hashTable = { 2, 6, SVGPathSegCurvetoQuadraticRelImpl__s_hashTableEntries, 5, SVGPathSegCurvetoQuadraticRelImpl__s_hashTableStrings};

} // namespace
