/* Automatically generated from impl/SVGZoomEventImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGZoomEventImpl__s_hashTableStrings[] = {
    "\0"
    "previousTranslate\0"
    "zoomRectScreen\0"
    "previousScale\0"
    "newTranslate\0"
    "newScale\0"
};


static const struct HashEntry SVGZoomEventImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGZoomEventImpl::PreviousTranslate, DontDelete|ReadOnly, 0, -1 },
   { 34, SVGZoomEventImpl::PreviousScale, DontDelete|ReadOnly, 0, -1 },
   { 19, SVGZoomEventImpl::ZoomRectScreen, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 48, SVGZoomEventImpl::NewTranslate, DontDelete|ReadOnly, 0, -1 },
   { 61, SVGZoomEventImpl::NewScale, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGZoomEventImpl::s_hashTable = { 2, 7, SVGZoomEventImpl__s_hashTableEntries, 7, SVGZoomEventImpl__s_hashTableStrings};

} // namespace
