<!DOCTYPE kpartgui>
<kpartgui version="2" name="kcoloredit">
<ActionProperties>
  <Action name="color_from_palette" icon="colorize"/>
  <Action name="color_from_screen" icon="colorpicker"/>
</ActionProperties>

<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_new_window" append="new_merge"/>
  </Menu>
  <Menu name="color"><text>&amp;Color</text>
    <Action name="color_from_palette"/>
    <Action name="color_from_screen"/>
  </Menu>
  <Menu name="settings">
    <Action name="color_view_names" append="show_merge"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_new" />
  <Action name="file_open" />
  <Action name="file_save" />
  <Action name="color_from_screen"/>
</ToolBar>
</kpartgui>
