<!DOCTYPE kpartgui>
<kpartgui name="KViewPart" version="12">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
   <Action name="file_open"/>
   <Action name="file_open_recent"/>
   <Action name="file_save" group="save_merge"/>
   <Action name="file_save_as" group="save_merge"/>
   <Separator group="save_merge"/>
   <Action name="file_print" group="print_merge"/>
   <Menu name="export" group="print_merge"><text>Export As</text>
     <Action name="export_text"/>
     <DefineGroup name="export_merge" append="export_merge"/>
   </Menu>
   <Separator group="print_merge"/>
 </Menu>
 <Menu name="edit"><text>&amp;Edit</text>
   <Action name="move_tool" group="tools_merge"/>
   <Action name="selection_tool" group="tools_merge"/>
   <Separator group="tools_merge"/>
   <Action name="copy_text"/>
   <Action name="edit_select_all"/>
   <Action name="edit_deselect_all"/>
   <Separator/>
   <Action name="find"/>
   <Action name="findnext"/>
   <Action name="findprev"/>
   <Merge/>
 </Menu>
 <Menu name="view"><text>&amp;View</text>
  <Action name="viewmode"/>
  <Merge/>
  <Separator/>
  <Action name="view_media"/>
  <Action name="view_orientation"/>
  <Action name="view_use_document_specified_size"/>
  <Separator/>
  <Action name="view_zoom_in"/>
  <Action name="view_zoom_out"/>
  <Separator/>
  <Action name="view_fit_to_page"/>
  <Action name="view_fit_to_width"/>
  <Action name="view_fit_to_height"/>
 </Menu>
  <Menu name="go"><text>&amp;Go</text>
    <Action name="go_goto_page"/>
    <Separator/>
    <Action name="go_previous"/>
    <Action name="go_next"/>
    <Separator/>
    <Action name="go_first"/>
    <Action name="go_last"/>
    <Separator/>
    <Action name="go_read_up"/>
    <Action name="go_read_down"/>
    <Separator/>
    <Action name="go_back"/>
    <Action name="go_forward"/>
  </Menu>
 <Menu name="settings">
  <Action name="show_sidebar" group="settings_top"/>
  <Action name="scrollbarHandling" group="settings_top"/>
  <Action name="watch_file" group="additional_settings_merge"/>
  <Action name="options_configure" group="configure_merge"/>
 </Menu>  
 <Menu name="help"><text>&amp;Help</text>
    <Action name="help_about_kviewshell" group="help_about"/>
  </Menu>
 
</MenuBar>
<ToolBar name="mainToolBar">
  <Action name="file_print"/>
  <Separator/>
  <Action name="go_first"/>
  <Action name="go_previous"/>
  <Action name="go_next"/>
  <Action name="go_last"/>
  <Action name="readDown"/>
  <Separator/>
  <Action name="view_zoom_in"/>
  <Action name="view_zoom"/>
  <Action name="view_zoom_out"/>
  <Separator/>
  <Action name="move_tool"/>
  <Action name="selection_tool"/>
  <Separator/>
  <Action name="go_back"/>
  <Action name="go_forward"/>
</ToolBar>
</kpartgui>

