//-*-C++-*-
/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2000-2003 by Andreas Zehender
    email                : zehender@kde.org
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/


#ifndef PMALLOBJECTS_H
#define PMALLOBJECTS_H

#include "pmscene.h"
#include "pmglobalsettings.h"

#include "pmbox.h"
#include "pmsphere.h"
#include "pmcylinder.h"
#include "pmcone.h"
#include "pmtorus.h"
#include "pmlathe.h"
#include "pmprism.h"
#include "pmsor.h"
#include "pmsqe.h"
#include "pmheightfield.h"
#include "pmtext.h"
#include "pmjuliafractal.h"

#include "pmblob.h"
#include "pmblobsphere.h"
#include "pmblobcylinder.h"

#include "pmplane.h"
#include "pmpolynom.h"

#include "pmdeclare.h"
#include "pmobjectlink.h"

#include "pmcsg.h"

#include "pmdisc.h"
#include "pmbicubicpatch.h"
#include "pmtriangle.h"

#include "pmboundedby.h"
#include "pmclippedby.h"

#include "pmcamera.h"

#include "pmtranslate.h"
#include "pmscale.h"
#include "pmrotate.h"
#include "pmpovraymatrix.h"

#include "pmlight.h"
#include "pmlookslike.h"
#include "pmprojectedthrough.h"

#include "pmtexture.h"
#include "pmpigment.h"
#include "pmsolidcolor.h"
#include "pmlistpattern.h"
#include "pmquickcolor.h"
#include "pmnormal.h"
#include "pmfinish.h"
#include "pmpattern.h"
#include "pmblendmapmodifiers.h"
#include "pmimagemap.h"
#include "pmbumpmap.h"
#include "pmtexturemap.h"
#include "pmmaterialmap.h"
#include "pmwarp.h"
#include "pmslope.h"
#include "pmdensity.h"

#include "pmskysphere.h"
#include "pmrainbow.h"
#include "pmfog.h"
#include "pminterior.h"
#include "pmmedia.h"
#include "pmmaterial.h"

#include "pmcomment.h"
#include "pmraw.h"

// POV-Ray 3.5 objects
#include "pmisosurface.h"
#include "pmradiosity.h"
#include "pmglobalphotons.h"
#include "pmphotons.h"
#include "pmlightgroup.h"
#include "pminteriortexture.h"
#include "pmspheresweep.h"
#include "pmmesh.h"

#endif
