/* Automatically generated from impl/SVGTextPositioningElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGTextPositioningElementImpl__s_hashTableStrings[] = {
    "\0"
    "rotate\0"
    "dx\0"
    "dy\0"
};


static const struct HashEntry SVGTextPositioningElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 9, SVGTextPositioningElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 12, SVGTextPositioningElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 8, SVGTextPositioningElementImpl::Dx, DontDelete|ReadOnly, 0, -1 },
   { 11, SVGTextPositioningElementImpl::Dy, DontDelete|ReadOnly, 0, 7 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGTextPositioningElementImpl::Rotate, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGTextPositioningElementImpl::s_hashTable = { 2, 8, SVGTextPositioningElementImpl__s_hashTableEntries, 7, SVGTextPositioningElementImpl__s_hashTableStrings};

} // namespace
