<!DOCTYPE kpartgui>
<kpartgui name="KViewShell" version="5">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
   <Action name="reload" append="open_merge" />
   <DefineGroup name="save_merge" append="save_merge" />
   <DefineGroup name="info_merge" append="save_merge"/>
   <DefineGroup name="print_merge" append="save_merge" />
 </Menu>
 <Menu name="edit"><text>&amp;Edit</text>
   <DefineGroup name="tools_merge"/>
 </Menu>
 <Menu name="view"><text>&amp;View</text>
   <DefineGroup name="view_top"/>
 </Menu>
 <Menu name="go"><text>&amp;Go</text>
   <DefineGroup name="go_top"/>
 </Menu>
 <Merge/>
 <Menu name="settings"><text>&amp;Settings</text>
  <DefineGroup name="settings_top" append="show_merge"/>
  <Separator append="show_merge"/>
  <DefineGroup name="additional_settings_merge" append="show_merge"/>
  <DefineGroup name="configure_merge" append="configure_merge" />
  <Merge/>
 </Menu>
  <Menu name="help"><text>&amp;Help</text>
    <DefineGroup name="help_top"/>
    <DefineGroup name="help_about"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <Action name="reload"/>
</ToolBar>
<StatusBar/>
</kpartgui>

