/* Automatically generated from impl/SVGCursorElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGCursorElementImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGCursorElementImpl__s_hashTableEntries[] = {
   { 1, SVGCursorElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 3, SVGCursorElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGCursorElementImpl::s_hashTable = { 2, 3, SVGCursorElementImpl__s_hashTableEntries, 3, SVGCursorElementImpl__s_hashTableStrings};

} // namespace
