<!DOCTYPE kpartgui>
<kpartplugin name="kviewtemplate" library="kview_templateplugin">
	<MenuBar>
		<Menu name="tools"><Text>&amp;Tools</Text>
			<Action name="plugin_template"/>
		</Menu>
	</MenuBar>
	<ToolBar name="extraToolBar">
		<Action name="plugin_template"/>
	</ToolBar>
</kpartplugin>
