<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<!--
SYNC: Do not change the number of quotes before the version number
      - it is parsed by the KolourPaint wrapper script.
-->
<gui name="kolourpaint" version="24">

<!--
SYNC: Check for duplicate actions in menus caused by some of our actions
      being added to ui_standards.rc.  Makes me wonder why we are using
      merging in the first place.
-->

<MenuBar>
    <Menu name="file">
        <!-- <Action name="file_new_window" append="new_merge" /> -->

        <Action name="file_export" append="save_merge" />
        <Separator append="save_merge" />

        <Action name="file_set_as_wallpaper_centered" />
        <Action name="file_set_as_wallpaper_tiled" />
    </Menu>

    <Menu name="edit">
        <Action name="edit_paste_in_new_window" append="edit_paste_merge" />

        <Action name="edit_copy_to_file" />
        <Action name="edit_paste_from_file" />
    </Menu>

    <!-- SRC: ui_standards.rc v10 (KDE 3.3) -->
    <Menu name="view" noMerge="1"><text>&amp;View</text>
        <Action name="view_actual_size"/>
        <Action name="view_fit_to_page"/>
        <Action name="view_fit_to_width"/>
        <Action name="view_fit_to_height"/>

        <Separator />

        <!-- <MergeLocal name="view_zoom_merge"/> -->
        <Action name="view_zoom_in"/>

        <!-- Changed from "view_zoom" to allow custom ordering of zoom
             actions in "mainToolBar" (which has a hardcoded position for
             "view_zoom").
          -->
        <Action name="view_zoom_to"/>

        <Action name="view_zoom_out"/>

        <WeakSeparator/>

        <Action name="view_redisplay"/>

        <Separator/>

        <!-- <MergeLocal/> -->
        <Action name="view_show_grid" />
        <Action name="view_show_thumbnail" />

        <Separator/>

        <Action name="view_zoomed_thumbnail" />
        <Action name="view_show_thumbnail_rectangle" />
    </Menu>

    <Menu name="image"><text>&amp;Image</text>
        <Action name="image_crop" />
        <Action name="image_auto_crop" />
        <Separator />
        <Action name="image_resize_scale" />
        <Action name="image_flip" />
        <Action name="image_rotate" />
        <Action name="image_skew" />
        <Separator />
        <Action name="image_convert_to_black_and_white" />
        <Action name="image_convert_to_grayscale" />
        <Action name="image_more_effects" />
        <Separator />
        <Action name="image_invert_colors" />
        <Action name="image_clear" />
    </Menu>

    <Menu name="settings">
        <Action name="settings_show_path" append="show_merge" />
    </Menu>

    <Menu name="help">
        <Action name="help_taking_screenshots" />
    </Menu>

</MenuBar>

<ToolBar name="mainToolBar">
    <Action name="view_zoom_in" />
    <Action name="view_zoom_out" />
    <Action name="view_zoom_to" />
</ToolBar>

<ToolBar name="textToolBar" fullWidth="false" position="top" hidden="true"><text>Text Toolbar</text>
    <Action name="text_font_family" />
    <Action name="text_font_size" />
    <Separator />
    <Action name="text_bold" />
    <Action name="text_italic" />
    <Action name="text_underline" />
    <Action name="text_strike_thru" />
</ToolBar>

<Menu name="selectionToolRMBMenu"><text>Selection Tool RMB Menu</text>
    <!-- SRC: ui_standards.rc v10 (KDE 3.3) -->
    <!-- <Menu name="edit"><text>&amp;Edit</text> -->

        <!-- <Action name="edit_undo"/>
        <Action name="edit_redo"/>
        <MergeLocal name="edit_undo_merge"/>
        <Separator/> -->
        <Action name="edit_cut"/>
        <Action name="edit_copy"/>
        <Action name="edit_paste"/>
        <!-- CUSTOM -->  <!-- <Action name="edit_paste_in_new_window" /> -->
        <MergeLocal name="edit_paste_merge"/>
        <Action name="edit_clear"/>
        <!-- <Separator/> -->
        <Action name="edit_select_all"/>
        <!-- <Action name="edit_deselect"/>  -->
        <MergeLocal name="edit_select_merge"/>
        <Separator/>
        <Action name="edit_find"/>
        <Action name="edit_find_next"/>
        <Action name="edit_find_last"/>
        <Action name="edit_replace"/>
        <MergeLocal name="edit_find_merge"/>
        <Separator/>
        <!-- CUSTOM -->  <Action name="edit_copy_to_file" />
        <!-- CUSTOM -->  <Action name="edit_paste_from_file" />
        <MergeLocal/>

    <!-- </Menu> -->


    <Separator/>


    <!-- <Menu name="image"><text>&amp;Image</text> -->

        <Action name="image_crop" />
        <!-- <Action name="image_auto_crop" /> -->
        <Separator />
        <Action name="image_resize_scale" />
        <Action name="image_flip" />
        <Action name="image_rotate" />
        <Action name="image_skew" />
        <Separator />
        <!-- <Action name="image_convert_to_black_and_white" /> -->
        <!-- <Action name="image_convert_to_grayscale" /> -->
        <!-- <Action name="image_more_effects" />  -->
        <!-- <Separator /> -->
        <Action name="image_invert_colors" />
        <!-- <Action name="image_clear" /> -->

    <!-- </Menu> -->
</Menu>

</gui>
