/* Automatically generated from impl/SVGPathSegImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KSVG {

static const char SVGPathSegImpl__s_hashTableStrings[] = {
    "\0"
    "pathSegTypeAsLetter\0"
    "pathSegType\0"
};


static const struct HashEntry SVGPathSegImpl__s_hashTableEntries[] = {
   { 21, SVGPathSegImpl::PathSegType, DontDelete|ReadOnly, 0, 3 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegImpl::PathSegTypeAsLetter, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPathSegImpl::s_hashTable = { 2, 4, SVGPathSegImpl__s_hashTableEntries, 3, SVGPathSegImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KSVG {

static const char SVGPathSegImplConstructor__s_hashTableStrings[] = {
    "\0"
    "PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS\0"
    "PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL\0"
    "PATHSEG_CURVETO_CUBIC_SMOOTH_ABS\0"
    "PATHSEG_CURVETO_CUBIC_SMOOTH_REL\0"
    "PATHSEG_CURVETO_QUADRATIC_ABS\0"
    "PATHSEG_CURVETO_QUADRATIC_REL\0"
    "PATHSEG_LINETO_HORIZONTAL_ABS\0"
    "PATHSEG_LINETO_HORIZONTAL_REL\0"
    "PATHSEG_LINETO_VERTICAL_ABS\0"
    "PATHSEG_LINETO_VERTICAL_REL\0"
    "PATHSEG_CURVETO_CUBIC_ABS\0"
    "PATHSEG_CURVETO_CUBIC_REL\0"
    "PATHSEG_LINETO_ABS\0"
    "PATHSEG_LINETO_REL\0"
    "PATHSEG_MOVETO_ABS\0"
    "PATHSEG_MOVETO_REL\0"
    "PATHSEG_CLOSEPATH\0"
    "PATHSEG_ARC_ABS\0"
    "PATHSEG_ARC_REL\0"
    "PATHSEG_UNKNOWN\0"
};


static const struct HashEntry SVGPathSegImplConstructor__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 317, KSVG::PATHSEG_CURVETO_CUBIC_ABS, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 171, KSVG::PATHSEG_CURVETO_QUADRATIC_REL, DontDelete|ReadOnly, 0, 27 },
   { 479, KSVG::PATHSEG_ARC_REL, DontDelete|ReadOnly, 0, -1 },
   { 495, KSVG::PATHSEG_UNKNOWN, DontDelete|ReadOnly, 0, 23 },
   { 369, KSVG::PATHSEG_LINETO_ABS, DontDelete|ReadOnly, 0, -1 },
   { 108, KSVG::PATHSEG_CURVETO_CUBIC_SMOOTH_REL, DontDelete|ReadOnly, 0, 28 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 426, KSVG::PATHSEG_MOVETO_REL, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 343, KSVG::PATHSEG_CURVETO_CUBIC_REL, DontDelete|ReadOnly, 0, 24 },
   { 463, KSVG::PATHSEG_ARC_ABS, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 75, KSVG::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS, DontDelete|ReadOnly, 0, -1 },
   { 231, KSVG::PATHSEG_LINETO_HORIZONTAL_REL, DontDelete|ReadOnly, 0, -1 },
   { 388, KSVG::PATHSEG_LINETO_REL, DontDelete|ReadOnly, 0, -1 },
   { 38, KSVG::PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL, DontDelete|ReadOnly, 0, -1 },
   { 407, KSVG::PATHSEG_MOVETO_ABS, DontDelete|ReadOnly, 0, -1 },
   { 445, KSVG::PATHSEG_CLOSEPATH, DontDelete|ReadOnly, 0, 25 },
   { 141, KSVG::PATHSEG_CURVETO_QUADRATIC_ABS, DontDelete|ReadOnly, 0, 26 },
   { 201, KSVG::PATHSEG_LINETO_HORIZONTAL_ABS, DontDelete|ReadOnly, 0, -1 },
   { 261, KSVG::PATHSEG_LINETO_VERTICAL_ABS, DontDelete|ReadOnly, 0, -1 },
   { 289, KSVG::PATHSEG_LINETO_VERTICAL_REL, DontDelete|ReadOnly, 0, -1 },
   { 1, KSVG::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPathSegImplConstructor::s_hashTable = { 2, 29, SVGPathSegImplConstructor__s_hashTableEntries, 23, SVGPathSegImplConstructor__s_hashTableStrings};

} // namespace
